@@ -115,9 +115,13 @@ void no_colors(void)
 
 /* display usage and exit */
 static void usage(int status, const char *flags, struct option const opts[],
-                  const char * const help[], int blabber)
+                  const char * const help[], const char *desc, int blabber)
 {
-	unsigned long i;
+	const char opt_arg[] = "[arg]";
+	const char a_arg[] = "<arg>";
+	size_t a_arg_len = strlen(a_arg) + 1;
+	size_t i, optlen;
+
 	if (status != EXIT_SUCCESS)
 		dup2(STDERR_FILENO, STDOUT_FILENO);
 	if (blabber == 0) {
@@ -132,14 +136,26 @@ static void usage(int status, const char *flags, struct option const opts[],
 					DKBLUE, applets[i].opts, NORM,
 					RED, NORM, _(applets[i].desc));
 	} else if (blabber > 0) {
-			printf("%sUsage:%s %s%s%s <opts> %s%s%s %s:%s %s\n", GREEN, NORM,
-				YELLOW, applets[blabber].name, NORM,
-				DKBLUE, applets[blabber].opts, NORM,
-				RED, NORM, _(applets[blabber].desc));
+		printf("%sUsage:%s %s%s%s [opts] %s%s%s %s:%s %s\n",
+			GREEN, NORM,
+			YELLOW, applets[blabber].name, NORM,
+			DKBLUE, applets[blabber].opts, NORM,
+			RED, NORM, _(applets[blabber].desc));
+		if (desc)
+			printf("\n%s\n", desc);
 	}
 	if (module_name != NULL)
 		printf("%sLoaded module:%s\n%s%8s%s %s<args>%s\n", GREEN, NORM, YELLOW, module_name, NORM, DKBLUE, NORM);
 
+	/* Prescan the --long opt length to auto-align. */
+	optlen = 0;
+	for (i = 0; opts[i].name; ++i) {
+		size_t l = strlen(opts[i].name);
+		if (opts[i].has_arg != no_argument)
+			l += a_arg_len;
+		optlen = MAX(l, optlen);
+	}
+
 	printf("\n%sOptions:%s -[%s]\n", GREEN, NORM, flags);
 	for (i = 0; opts[i].name; ++i) {
 		/* this assert is a life saver when adding new applets. */
@@ -153,11 +169,14 @@ static void usage(int status, const char *flags, struct option const opts[],
 
 		/* then the long flag + help text */
 		if (opts[i].has_arg == no_argument)
-			printf("--%-15s%s*%s %s\n", opts[i].name,
+			printf("--%-*s %s*%s %s\n", (int)optlen, opts[i].name,
 				RED, NORM, _(help[i]));
 		else
-			printf("--%-8s %s<arg>%s %s*%s %s\n", opts[i].name,
-				DKBLUE, NORM, RED, NORM, _(help[i]));
+			printf("--%s %s%s%s%*s %s*%s %s\n",
+				opts[i].name,
+				DKBLUE, (opts[i].has_arg == a_argument ? a_arg : opt_arg), NORM,
+				(int)(optlen - strlen(opts[i].name) - a_arg_len), "",
+				RED, NORM, _(help[i]));
 	}
 	exit(status);
 }
@@ -960,7 +979,8 @@ initialize_flat(const char *overlay, int cache_type, bool force)
 	}
 
 	if (!quiet)
-		warn("Updating ebuild %scache ... ", cache_type == CACHE_EBUILD ? "" : "meta");
+		warn("Updating ebuild %scache in %s ... ",
+			cache_type == CACHE_EBUILD ? "" : "meta", overlay);
 
 	count = frac = secs = 0;
 
